@@ -1,4 +1,4 @@
-/* $Id: config.cc,v 1.3 2004/04/04 20:40:00 murrayc Exp $ */
+/* $Id: config.cc 808 2005-10-20 06:54:43Z murrayc $ */
 // -*- C++ -*- // this is for the .hg, I realize gensig puts one in
 
 /* config.cc
@@ -32,6 +32,11 @@ namespace Gda
 namespace Config
 {
 
+void DataSourceInfoTraits::release_c_type(CType ptr)
+{
+  gda_data_source_info_free(const_cast<CTypeNonConst>(ptr));
+}
+
 Glib::ustring get_value_string(const Glib::ustring& path)
 {
   return gda_config_get_string(path.c_str());
@@ -145,9 +150,9 @@ Glib::RefPtr<DataModel> get_data_source_model()
 
 void save_data_source(const Glib::ustring& name, const Glib::ustring& provider,
                       const Glib::ustring& cnc_string, const Glib::ustring& description,
-                      const Glib::ustring& username, const Glib::ustring& password)
+                      const Glib::ustring& username, const Glib::ustring& password, bool is_global)
 {
-  gda_config_save_data_source(name.c_str(), provider.c_str(), cnc_string.c_str(), description.c_str(), username.c_str(), password.c_str());
+  gda_config_save_data_source(name.c_str(), provider.c_str(), cnc_string.c_str(), description.c_str(), username.c_str(), password.c_str(), is_global);
 }
 
 
