@@ -1,4 +1,4 @@
-/* $Id: value.cc 978 2007-02-04 12:20:31Z arminb $ */
+/* $Id: value.cc 1059 2007-03-30 13:40:09Z arminb $ */
 // -*- C++ -*- //
 
 /* value.cc
@@ -25,6 +25,19 @@
 #include <libgda/gda-enum-types.h>
 #include <libgda/gda-util.h>
 
+namespace
+{
+  // Changes the type of a given value
+  void value_reinit(GValue* value, GType g_type)
+  {
+    if(G_IS_VALUE(value) && G_VALUE_TYPE(value) != g_type)
+      g_value_unset(value);
+
+    if(!G_IS_VALUE(value))
+      g_value_init(value, g_type);
+  }
+}
+
 namespace Gnome
 {
   
@@ -51,112 +64,81 @@ Value Value::create_as_uint64(guint64 val)
 
 Value::Value(const guchar* val, long size)
 {
-  init(GDA_TYPE_BINARY);
-
   set(val, size);
 }
 
 Value::Value(const GdaBlob *val)
 {
-  init(GDA_TYPE_BLOB);
   set(val);
 }
 
 Value::Value(bool val)
 {
-  init( Glib::Value<bool>::value_type() );
-
   set(val);
 }
 
 Value::Value(const Glib::Date& val)
 {
-  init( Glib::Value<Glib::Date>::value_type() );
-
   set(val);
 }
 
 Value::Value(double val)
 {
-  init( Glib::Value<double>::value_type() );
-
   set(val);
 }
 
 Value::Value(const GeometricPoint& val)
 {
-  init( GDA_TYPE_GEOMETRIC_POINT );
-
   set(val);
 }
 
 Value::Value(int val)
 {
-  init( Glib::Value<int>::value_type() );
-
   set(val);
 }
 
 Value::Value(const GdaValueList* val)
 {
-  init(GDA_TYPE_LIST);
-
   set(val);
 }
 
 Value::Value(const GdaNumeric* val)
 {
-  init(GDA_TYPE_NUMERIC);
-
   set(val);
 }
 
 Value::Value(float val)
 {
-  init( Glib::Value<float>::value_type() );
-
   set(val);
 }
 
 Value::Value(gshort val)
 {
-  init(GDA_TYPE_SHORT);
-
   set(val);
 }
 
 Value::Value(gushort val)
 {
-  init(GDA_TYPE_USHORT);
-
   set(val);
 }
 
 Value::Value(const Glib::ustring& val)
 {
-  init( Glib::Value<Glib::ustring>::value_type() );
-
   set(val);
 }
 
 Value::Value(const char* val)
 {
-  init( Glib::Value<Glib::ustring>::value_type() );
-
   set(val);
 }
 
 Value::Value(const Time& val)
 {
-  init( GDA_TYPE_TIME );
-
   set(val);
 }
 
 Value::Value(const Timestamp& val)
 {
-  init( GDA_TYPE_TIMESTAMP );
-
   set(val);
 }
 
@@ -175,7 +157,7 @@ Value Value::create_as_time_t(time_t val)
 
 Value::Value(guint val)
 {
-  init( Glib::Value<guint>::value_type() );
+  set(val);
 }
 
 /*
@@ -187,13 +169,13 @@ Value::Value(const Glib::ustring& as_string, GType type)
 
 bool Value::operator==(const Value& src) const
 {
-  if( !gobj() && !src.gobj()) //If both are null.
+  if( !G_IS_VALUE(gobj()) && !G_IS_VALUE(src.gobj())) //If both are null.
     return true;
 
-  if( !gobj() && src.gobj()) //If one is null.
+  if( !G_IS_VALUE(gobj()) && G_IS_VALUE(src.gobj())) //If one is null.
     return false;
 
-  if( gobj() && !src.gobj()) //If one is null.
+  if( G_IS_VALUE(gobj()) && !G_IS_VALUE(src.gobj())) //If one is null.
     return false;
 
   if(get_value_type() != src.get_value_type()) //gda_value_compare() can only compare GValues of the same type.
@@ -216,23 +198,32 @@ Value::Value()
 
 Value::Value(const Value& src)
 {
-  init(src.gobj());
+  if(G_IS_VALUE(src.gobj()))
+    init(src.gobj());
 }
 
 Value::Value(const GValue* castitem)
 {
-  init(castitem);
+  if(G_IS_VALUE(castitem))
+    init(castitem);
 }
 
 Value& Value::operator=(const Value& src)
 {
-  init(src.gobj()); //TODO: We probably shouldn't allow this to be called twice.
+  // Unset current value, if any
+  if(G_IS_VALUE(gobj())) g_value_unset(gobj());
+  // Set new value, if any
+  if(G_IS_VALUE(src.gobj())) init(src.gobj());
 
   return *this;
 }
 
 Value::~Value()
 {
+  // If the value does not have a type, we just set one so that the
+  // ValueBase destructor does not complain.
+  if(!G_IS_VALUE(gobj()))
+    init(G_TYPE_INT);
 }
 
 GType Value::get_value_type() const
@@ -255,9 +246,10 @@ gint64 Value::get_int64() const
   return g_value_get_int64(const_cast<GValue*>(gobj()));
 }
 
-void Value::set(gint64 val)
+void Value::set_int64(gint64 val)
 {
-  g_value_set_int64(gobj(), val); 
+  value_reinit(gobj(), G_TYPE_INT64);
+  g_value_set_int64(gobj(), val);
 }
 
 guint64 Value::get_uint64() const
@@ -265,8 +257,9 @@ guint64 Value::get_uint64() const
   return g_value_get_uint64(const_cast<GValue*>(gobj()));
 }
 
-void Value::set(guint64 val)
+void Value::set_uint64(guint64 val)
 {
+  value_reinit(gobj(), G_TYPE_UINT64);
   g_value_set_uint64(gobj(), val); 
 }
 
@@ -308,6 +301,7 @@ bool Value::get_boolean() const
 
 void Value::set(bool val)
 {
+  value_reinit(gobj(), Glib::Value<bool>::value_type());
   g_value_set_boolean(gobj(), static_cast<int>(val)); 
 }
 
@@ -323,6 +317,7 @@ Glib::Date Value::get_date() const
 
 void Value::set(const Glib::Date& val)
 {
+  value_reinit(gobj(), Glib::Value<Glib::Date>::value_type());
   g_value_set_boxed(gobj(), val.gobj());
 }
 
@@ -333,6 +328,7 @@ double Value::get_double() const
 
 void Value::set(double val)
 {
+  value_reinit(gobj(), Glib::Value<double>::value_type());
   g_value_set_double(gobj(), val); 
 }
 
@@ -353,6 +349,7 @@ int Value::get_int() const
 
 void Value::set(int val)
 {
+  value_reinit(gobj(), Glib::Value<int>::value_type());
   g_value_set_int(gobj(), val); 
 }
 
@@ -383,6 +380,7 @@ float Value::get_float() const
 
 void Value::set(float val)
 {
+  value_reinit(gobj(), Glib::Value<float>::value_type());
   g_value_set_float(gobj(), val); 
 }
 
@@ -413,6 +411,7 @@ Glib::ustring Value::get_string() const
 
 void Value::set(const Glib::ustring& val)
 {
+  value_reinit(gobj(), G_TYPE_STRING);
   g_value_set_string(gobj(), val.c_str()); 
 }
 
@@ -448,6 +447,7 @@ guint Value::get_uint() const
 
 void Value::set(guint val)
 {
+  value_reinit(gobj(), Glib::Value<unsigned int>::value_type());
   g_value_set_uint(gobj(), val); 
 }
 
@@ -458,6 +458,7 @@ GType Value::get_g_type() const
 
 void Value::set(GType val)
 {
+  value_reinit(gobj(), G_TYPE_GTYPE);
   g_value_set_gtype(gobj(), val);
 }
 
@@ -540,7 +541,7 @@ const GType VALUE_TYPE_TIMESTAMP = GDA_TYPE_TIMESTAMP;
 */
 
 
-
+#if 0
 int value_compare_ext(const Glib::ValueBase& value1, const Glib::ValueBase& value2)
 {
   return gda_value_compare_ext(value1.gobj(), value2.gobj());
@@ -563,6 +564,7 @@ bool value_equal(const Glib::ValueBase& value1, const Glib::ValueBase& value2)
   int test = gda_value_compare(value1.gobj(), value2.gobj());
   return test == 0;
 }
+#endif
 
 
 
